function[pi4,pvalue] = Kurtosis_pi4(data)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code: Exercise 1.8(c)
% File: Kurtosis_pi4.m
%
% Bai-Ng (JBES, 2005) Kurtosis test statistics ``pi4''.
%
% INPUT:  data
% OUTPUT: pi4, pvalue
%
% Reference:
% Bai, J. and Ng, S. (2005).
%   Tests for skewness, kurtosis, and normality for time series 
%   data.
%   Journal of Business & Economic Statistics, 23(1), 49-60.
%   DOI: 10.1198/073500104000000271.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xx = data;
z  = xx-mean(xx);
n  = length(xx);
m2 = sum(z.*z)/(n-1);
m3 = sum(z.*z.*z)/(n-1);
m4 = sum(z.*z.*z.*z)/(n-1);
kappa  = 3;
a      = ones(3,1);
a(1,1) = 1;
a(2,1) = -4*m3;
a(3,1) = -2*m2*kappa;
vv     = [(z.*z.*z.*z-m2^2),z,(z.*z-m2)];
[nsize,nreg] = size(vv);
beta = zeros(nreg,nreg);

% Find the LS approximation solution with the QR decomposition
for i = 1:nreg
    A = vv(2:nsize,i);
    y = vv(1:nsize-1,i);
    if issparse(A),R = qr(A);
       else R = triu(qr(A));
    end
    b(i) = R\(R'\(A'*y));
    r    = y-A*b(i);
    rrr  = r'*r/nsize;
    se_b(i) = rrr;
end

bot = 0; top = 0;
for i = 1:nreg
    top = top+4*(b(i)^2)*se_b(i)^2/(1-b(i))^8;
    bot = bot+se_b(i)^2/((1-b(i))^4);
end

alpha = top/bot;
k = ceil(2.6614*(alpha*nsize)^(0.2));
if k > n/2; k = n/2;
end

% Variance-covariance matrix 
vcv = vv'*vv/(n-1);
for i = 1:k
    x = i/k;
    if abs(x) >= 0 && abs(x) <= .5;  % Weights Parzen window
       w = 1-6*(abs(x))^2+6*abs(x)^3;
    else
       w = 2*(1-abs(x))^3;
    end
    cov = vv(i+1:nsize,:)'*vv(1:nsize-i,:)/(n-1);
    vcv = vcv+w*cov;
    cov = vv(1:nsize-i,:)'*vv(i+1:nsize,:)/(n-1);
    vcv = vcv+w*cov;
end
d      = inv(eye(nreg)-beta');
omega  = d*vcv*d'; 
se     = sqrt(a'*omega*a/(m2^4))/sqrt(n);
stat3  = m4/(m2^2);
pi4    = (stat3-kappa)/se;  % end computing kurtosis test
pvalue = 1-normcdf(pi4,0,1);
